/**
 * \file pathTool.hpp
 *
 * Header file for the rednav path tool.
 *
 * \date 2011-01-07
 * \author redouane.boumghar@laas.fr
 * \ingroup rednav
 */


#ifndef REDNAV_LIBS_PATH_TOOL_HPP
#define REDNAV_LIBS_PATH_TOOL_HPP

// For the definition of jafar::lgl::NavPath
#include <lgl/NavGraph.hpp>
#include <lgl/Attributes.hpp>


namespace jafar {
namespace rednav {

	typedef enum { 
		PATH_TWO_POINTS=1,
		PATH_THREE_POINTS,
		PATH_FOUR_POINTS,
		PATH_COMMON_FILTER,   // Apply subsampling and max number of points
		PATH_RDV_CROP} PATH_TRANSFORMATION;

	class PathTool {
		private:
			/** Subsampling parameter in meters
			 *
			 * Only positive values. Zero means no subsampling.
			 */
			double subsampler;

			/** Indicate how many way points the output path can contain
			 *
			 * Only positive values. Zero means no filter on points number.
			 * 1 means only the first point after the start point so two real
			 * points.
			 *
			 */
			int maxwaypts;

		public:
			/** Default constructor to use tools separatly */
			PathTool();
			/** One Shot constructor to apply a path transformation on a given path */
			PathTool(jafar::lgl::NavPath&, PATH_TRANSFORMATION);

			/** Destructor */
			~PathTool();

			/** Generic Tool to make easy generic calls */
			bool applyTool(jafar::lgl::NavPath& src, jafar::lgl::NavPath& out, PATH_TRANSFORMATION);

			/** Two points transformation */
			bool twoPoints(jafar::lgl::NavPath& src, jafar::lgl::NavPath& out);

			/** Path subsampling */
			bool subSample(jafar::lgl::NavPath& src, jafar::lgl::NavPath& out, double subsamp);

			/** Set the max number of way points */
			void setMaxWayPts(int n);


	};
}} // end of namespaces

#endif //REDNAV_LIBS_PATH_TOOL_HPP
