/**
 * \file pathTool.cpp
 *
 * Source file for the rednav path tool.
 *
 * \date 2011-01-07
 * \author redouane.boumghar@laas.fr
 * \ingroup rednav
 */

#include <rednav/pathTool.hpp>

namespace jafar {
namespace rednav {

	// Constructor that does nothing but giving calls to tools
	PathTool::PathTool()
	{ // {{{
		subsampler = 0.0;
		maxwaypts  = 0;
	} // }}}

	// Tooled Constructor
	PathTool::PathTool(jafar::lgl::NavPath& path, PATH_TRANSFORMATION trsf)
	{ // {{{
		subsampler = 0.0;
		maxwaypts  = 0;
	} // }}}

	//-------------------------- Destructor ---------
	PathTool::~PathTool()
	{ // {{{
	} // }}}

	bool PathTool::applyTool(jafar::lgl::NavPath& inpath, jafar::lgl::NavPath& outpath, PATH_TRANSFORMATION)
	{//{{{
		return true;
	}//}}}

	bool PathTool::twoPoints(jafar::lgl::NavPath& foundPath, jafar::lgl::NavPath& tPath)
	{//{{{
		jafar::lgl::Attributes emptyAtt;
		// Init.
		jafar::lgl::NavPath::const_iterator path_it;
		jafar::lgl::NavPath::const_iterator path_it2;

		// Temporary pointers to the start and goal points
		path_it = foundPath.begin();
		if (path_it == foundPath.end()) { return false; }
		path_it2 = foundPath.end(); --path_it2;
		if (path_it2 == foundPath.end()) { return false; }

		//---------- Prepare two-points ways for Ressac v1
		// Ressac v1 : ressac only able to follow two points for mapping
		//----- Find the farthest points to the line start-goal on each side
		// start-goal vector coordinates
		double vx_sg = (double)(((*path_it2).first).pos_x - ((*path_it).first).pos_x);
		double vy_sg = (double)(((*path_it2).first).pos_y - ((*path_it).first).pos_y);
		// 'Absolute' start coordinates
		double co_sx = (double)((*path_it).first).pos_x;
		double co_sy = (double)((*path_it).first).pos_y;
		// Relative Goal coordinates with transformation Start --> (0,0)
		double co_gx = (double)((*path_it2).first).pos_x - co_sx;
		double co_gy = (double)((*path_it2).first).pos_y - co_sy;
		// Considered path point coordinates
		double co_ax = 0.0;
		double co_ay = 0.0;
		// 'Absolute' On-DirectLine projected points coordinates
		double co_hx = 0.0;
		double co_hy = 0.0;
		// Slope coefficient of the direct line between start and goal points
		bool mslope_vertical = false;
		double mslope   = 0.0;
		double mslope_c = 0.0;
		if (vx_sg != 0.0) 
		{
			mslope = vy_sg / vx_sg;
			mslope_c = co_gx + mslope * co_gy;
		} else {
			mslope_vertical =  true;
		}
		// Distance parameters
		double dist = 0.0;
		double maxdist_a = 0.0;
		double maxdist_b = 0.0;
		// Axis relative signed position
		double sign_a = 0.0; 

		// To tell if we have found at least one point on the positive side
		int plus_ni  = -1;
		double plus_ax = 0.0;
		double plus_ay = 0.0;
		// To tell if we have found at least one point on the negative side
		int minus_ni = -1;
		double minus_ax = 0.0;
		double minus_ay = 0.0;
		
		int N = 0;
		for( path_it  = foundPath.begin(); 
			 path_it != foundPath.end();
			 ++path_it)
		{
			co_ax = (double)((*path_it).first).pos_x ;
			co_ay = (double)((*path_it).first).pos_y ;
			if (mslope_vertical)
			{
				co_hx = co_sx;
				co_hy = co_ay;
			} else {
				// Getting relative position when start --> (0,0)
				co_ax = co_ax - co_sx;
				co_ay = co_ay - co_sy;
				// Calculating relative projection
				co_hx = (co_ax * co_gx + co_ay * co_gy) / mslope_c;
				co_hy = mslope * co_hx;
				// Translating to the main frame
				co_ax = co_ax + co_sx;
				co_ay = co_ay + co_sy;
				co_hx = co_hx + co_sx;
				co_hy = co_hy + co_sy;
			}

			dist = sqrt( (co_hx-co_ax) * (co_hx-co_ax) 
					   + (co_hy-co_ay) * (co_hy-co_ay) ) ;
			// Saving farthest point on its side
			sign_a = (co_hx - co_ax)*(co_gy)
				   + (co_hy - co_ay)*(co_gx);
			if ( sign_a > 0.0 )
			{
				if (dist > maxdist_a)
				{
					maxdist_a = dist;
					plus_ax = co_ax;
					plus_ay = co_ay;
					plus_ni = N;
				}
			} else {
				if (dist >= maxdist_b)
				{
					maxdist_b = dist;
					minus_ax = co_ax;
					minus_ay = co_ay;
					minus_ni = N;
				}
			}

			// Full path index count
			N++;
		}

		//----- Filling a 2-waypoints path
		tPath.clear();
		std::pair<jafar::lgl::NavLocation, jafar::lgl::Attributes> insertor;
		insertor.second = emptyAtt;
		if ( plus_ni >= 0 && minus_ni >= 0)
		{
			// There is a point on each side of the start-goal line
			insertor.first = jafar::lgl::NavLocation(plus_ax, plus_ay);
			tPath.push_back(insertor);
			insertor.first = jafar::lgl::NavLocation(minus_ax, minus_ay);
			tPath.push_back(insertor);
		} else {
			if (plus_ni >=0 && minus_ni == -1)
			{
				insertor.first = jafar::lgl::NavLocation(plus_ax, plus_ay);
				tPath.push_back(insertor);
				path_it = foundPath.begin();
				if (path_it != foundPath.end())
				{
					tPath.push_back((*path_it));
				} else {
					return false;
				}
			} else {
				if (minus_ni >=0 && plus_ni == -1)
				{
					insertor.first = jafar::lgl::NavLocation(minus_ax, minus_ay);
					tPath.push_back(insertor);
					path_it = foundPath.begin();
					if (path_it != foundPath.end())
					{
						tPath.push_back((*path_it));
					} else {
						return false;
					}
				} else {
					// No points found ? quitely impossible :)
					return false;
				}
			}
		}

		return true;
	}//}}}

	bool PathTool::subSample(jafar::lgl::NavPath& foundPath, jafar::lgl::NavPath& tPath, double subsamp)
	{//{{{
		/*
		//------------ variables
		double x_new = 0.0;
		double y_new = 0.0;
		double x_prevpos = 0.0;
		double y_prevpos = 0.0;
		double curvDist = 0.0;
		int N = -1;
		jafar::lgl::GraphPath<jafar::lgl::NavLocation>::const_iterator path_it;

		//--- TODO prepare two points ways for Ressac v1
		for(path_it = foundPath.begin(); 
			(N < GENPOS_MAX_TRAJECTORY_SIZE) && (path_it != foundPath.end());
			++path_it)
		{ 
			// Correcting coordinates in the superimposed LGL frame
			x_new = (double)(*path_it).pos_x;
			x_new = (x_new/2.0)*cellScale + xRMZero;	
			y_new = (double)(*path_it).pos_y; 
			y_new = (y_new/2.0)*cellScale + yRMZero;
			if (N==-1)
			{
				// Not using first point of the trajectory as it is the position of
				// the vehicule itself
				N=0;
			} else {
				// Resampling trajectory to a more loose one
				x_prevpos = x_prevpos - x_new;
				x_prevpos = x_prevpos*x_prevpos;
				y_prevpos = y_prevpos - y_new;
				y_prevpos = y_prevpos*y_prevpos;
				curvDist = curvDist + sqrt(x_prevpos + y_prevpos);
				if ( curvDist >= subsamp)
				{
					// Inverted coordinates to fit with regionMap/POM coordinates
					rednavCoopPath->points[N].x = y_new;
					rednavCoopPath->points[N].y = x_new;
					rednavCoopPath->points[N].theta = 0.0;
					//cout << "Path Point " << N << "  (" << y_new << ", " << x_new << ")" << endl;
					cout << y_new << "   " << x_new << "   " 
						 << (*path_it).pos_x << "   " << (*path_it).pos_y  << "   "
						 << ((*path_it).pos_x)/2 << "   " << -((*path_it).pos_y)/2 << endl;
					curvDist = 0.0;
					N++;
				} 
			}

			x_prevpos = x_new;
			y_prevpos = y_new;
		}  
		*/
		return false;
	}//}}}
}} // end of namespaces
